@@ -17,21 +17,15 @@
 
 """astroid packaging information"""
 
-distname = "astroid"
-
-modname = "astroid"
-
-version = "2.2.3"
+version = "2.3.1"
 numversion = tuple(int(elem) for elem in version.split(".") if elem.isdigit())
 
 extras_require = {}
 install_requires = [
-    "lazy_object_proxy",
-    "six",
-    "wrapt",
-    'typing;python_version<"3.5"',
-    'typed-ast>=1.3.0;python_version>="3.7" and implementation_name== "cpython"',
-    'typed-ast<1.3.0;python_version<"3.7" and implementation_name== "cpython"',
+    "lazy_object_proxy==1.4.*",
+    "six==1.12",
+    "wrapt==1.11.*",
+    'typed-ast>=1.4.0,<1.5;implementation_name== "cpython" and python_version<"3.8"',
 ]
 
 # pylint: disable=redefined-builtin; why license is a builtin anyway?
@@ -49,7 +43,6 @@ classifiers = [
     "Topic :: Software Development :: Quality Assurance",
     "Programming Language :: Python",
     "Programming Language :: Python :: 3",
-    "Programming Language :: Python :: 3.4",
     "Programming Language :: Python :: 3.5",
     "Programming Language :: Python :: 3.6",
     "Programming Language :: Python :: 3.7",
